==> viewer2d-0.0.1.tgz: package/src/transform2D.js <==
export default ({
  Matrix = DOMMatrixReadOnly,
  Quad = DOMQuad,
  drawArea = { width: 0, height: 0 },
  initTransform = new Matrix(),
  maxScale = Infinity,
  minScale = 0,
  fitViewRatio = 1
}) => {
  const core = {
    maxScale,
    minScale,
    fitViewRatio,
    drawArea,
    transform: initTransform,
    translate(dx = 0, dy = 0) {
      if (typeof dx !== "number" || typeof dy !== "number") {
        throw "dx and dy should be numbers.";
      }
      this.transform = new Matrix().translate(dx, dy).multiply(this.transform);
    },
    rotate(angle = 0, origin = { x: 0, y: 0 }) {
      if (typeof angle !== "number") {
        throw "Angle must be a number (degree)";
      }
      this.transform = this.transform.multiply(
        new Matrix()
          .translate(origin.x, origin.y)
          .rotate(angle)
          .translate(-origin.x, -origin.y)
      );
    },
    scale(factor = 1, origin = { x: 0, y: 0 }) {
      if (factor < 0) {
        throw "Scale factor should be > 0.";
      }
      let clampedFactor = factor;
      const scale = this.getScale();
      if (scale * clampedFactor >= maxScale) {
        clampedFactor = maxScale / scale;
      }
      if (scale * clampedFactor <= minScale) {
        clampedFactor = minScale / scale;
      }
      this.transform = this.transform.scale(
        clampedFactor,
        clampedFactor,
        1,
        origin.x,
        origin.y
      );
    },
    getTranslate(translateM = this.transform) {
      return { dx: translateM.e, dy: translateM.f };
    },
    getRotate(rotateM = this.transform) {
      return (
        (Math.atan2(rotateM.b / this.getScale(), rotateM.a / this.getScale()) *
          180) /
        Math.PI
      );
    },
    getScale(scaleM = this.transform) {
      return Math.sqrt(Math.pow(scaleM.a, 2) + Math.pow(scaleM.b, 2));
    },
    resetTranslate() {
      this.setState({ translate: { dx: 0, dy: 0 } });
    },
    resetRotate() {
      this.setState({ rotate: 0 });
    },
    resetScale() {
      this.setState({ scale: 1 });
    },
    setState({
      rotate = this.getRotate(),
      translate = this.getTranslate(),
      scale = this.getScale()
    } = {}) {
      if (typeof translate !== "object") {
        throw `'translate' argument should be an object, received '${typeof translate}'.`;
      }
      if (typeof rotate !== "number") {
        throw `'rotate' argument should be a number, received '${typeof rotate}'.`;
      }
      if (typeof scale !== "number") {
        throw `'scale' argument should be a number, received '${typeof scale}'.`;
      }
      const { dx = 0, dy = 0 } = translate;
      this.transform = new Matrix()
        .translate(dx, dy)
        .rotate(rotate)
        .scale(scale);
    },
    fitView(quad = {}, ratio = this.fitViewRatio) {
      this.transform = new Matrix().rotate(this.getRotate());
      const { x, y, width, height } = this.getViewRectFromQuad(quad);
      const quadCenter = {
        x: x + width / 2,
        y: y + height / 2
      };
      const drawAreaCenter = {
        x: this.drawArea.width / 2,
        y: this.drawArea.height / 2
      };
      const scaleFactor = Math.min(
        this.drawArea.width / width,
        this.drawArea.height / height
      );
      const translateTarget = {
        x: drawAreaCenter.x - quadCenter.x,
        y: drawAreaCenter.y - quadCenter.y
      };
      this.translate(translateTarget.x, translateTarget.y);
      this.scale(scaleFactor * ratio, this.getTransformedPoint(drawAreaCenter));
    },
    getTransformedPoint(point) {
      return this.transform.inverse().transformPoint(point);
    },
    getViewPoint(point) {
      return this.transform.transformPoint(point);
    },
    getViewRectFromQuad(quad = {}) {
      return new Quad(
        ...Object.values(quad).map(this.getViewPoint.bind(this))
      ).getBounds();
    }
  };

  return {
    translate(dx, dy) {
      return core.translate(dx, dy);
    },
    rotate(angle, origin) {
      return core.rotate(angle, core.getTransformedPoint(origin));
    },
    scale(factor, origin) {
      return core.scale(factor, core.getTransformedPoint(origin));
    },
    fitView(quad, ratio) {
      return core.fitView(quad, ratio);
    },
    getTransform() {
      return core.transform;
    }
  };
};

==> viewer2d-0.1.4.tgz: package/src/transform2D.js <==
export default ({
  Matrix = DOMMatrixReadOnly,
  Quad = DOMQuad,
  viewport = { width: 0, height: 0 },
  initTransform = new Matrix(),
  maxScale = Infinity,
  minScale = 0,
  fitViewRatio = 1,
}) => {
  const core = {
    maxScale,
    minScale,
    fitViewRatio,
    viewport,
    transform: initTransform,
    translate(dx = 0, dy = 0) {
      if (typeof dx !== "number" || typeof dy !== "number") {
        throw "dx and dy should be numbers.";
      }
      this.transform = new Matrix().translate(dx, dy).multiply(this.transform);
    },
    rotate(angle = 0, origin = { x: 0, y: 0 }) {
      if (typeof angle !== "number") {
        throw "Angle must be a number (degree)";
      }
      this.transform = this.transform.multiply(
        new Matrix()
          .translate(origin.x, origin.y)
          .rotate(angle)
          .translate(-origin.x, -origin.y)
      );
    },
    scale(factor = 1, origin = { x: 0, y: 0 }) {
      if (factor < 0) {
        throw "Scale factor should be > 0.";
      }
      let clampedFactor = factor;
      const scale = this.getScale();
      if (scale * clampedFactor >= maxScale) {
        clampedFactor = maxScale / scale;
      }
      if (scale * clampedFactor <= minScale) {
        clampedFactor = minScale / scale;
      }
      this.transform = this.transform.scale(
        clampedFactor,
        clampedFactor,
        1,
        origin.x,
        origin.y
      );
    },
    getTranslate(translateM = this.transform) {
      return { dx: translateM.e, dy: translateM.f };
    },
    getRotate(rotateM = this.transform) {
      return (
        (Math.atan2(rotateM.b / this.getScale(), rotateM.a / this.getScale()) *
          180) /
        Math.PI
      );
    },
    getScale(scaleM = this.transform) {
      return Math.sqrt(Math.pow(scaleM.a, 2) + Math.pow(scaleM.b, 2));
    },
    resetTranslate() {
      this.setState({ translate: { dx: 0, dy: 0 } });
    },
    resetRotate() {
      this.setState({ rotate: 0 });
    },
    resetScale() {
      this.setState({ scale: 1 });
    },
    setState({
      rotate = this.getRotate(),
      translate = this.getTranslate(),
      scale = this.getScale(),
    } = {}) {
      if (typeof translate !== "object") {
        throw `'translate' argument should be an object, received '${typeof translate}'.`;
      }
      if (typeof rotate !== "number") {
        throw `'rotate' argument should be a number, received '${typeof rotate}'.`;
      }
      if (typeof scale !== "number") {
        throw `'scale' argument should be a number, received '${typeof scale}'.`;
      }
      const { dx = 0, dy = 0 } = translate;
      this.transform = new Matrix()
        .translate(dx, dy)
        .rotate(rotate)
        .scale(scale);
    },
    fitView(quad = {}, ratio = this.fitViewRatio) {
      this.transform = new Matrix().rotate(this.getRotate());
      const { x, y, width, height } = this.getViewRectFromQuad(quad);
      const quadCenter = {
        x: x + width / 2,
        y: y + height / 2,
      };
      const viewportCenter = {
        x: this.viewport.width / 2,
        y: this.viewport.height / 2,
      };
      const scaleFactor = Math.min(
        this.viewport.width / width,
        this.viewport.height / height
      );
      const translateTarget = {
        x: viewportCenter.x - quadCenter.x,
        y: viewportCenter.y - quadCenter.y,
      };
      this.translate(translateTarget.x, translateTarget.y);
      this.scale(scaleFactor * ratio, this.getTransformedPoint(viewportCenter));
    },
    getTransformedPoint(point) {
      return this.transform.inverse().transformPoint(point);
    },
    getViewPoint(point) {
      return this.transform.transformPoint(point);
    },
    getViewRectFromQuad(quad = {}) {
      return new Quad(
        ...Object.values(quad).map(this.getViewPoint.bind(this))
      ).getBounds();
    },
  };

  return {
    translate(dx, dy) {
      return core.translate(dx, dy);
    },
    rotate(angle, origin) {
      return core.rotate(angle, core.getTransformedPoint(origin));
    },
    scale(factor, origin) {
      return core.scale(factor, core.getTransformedPoint(origin));
    },
    fitView(quad, ratio) {
      return core.fitView(quad, ratio);
    },
    getTransform() {
      return core.transform;
    },
  };
};
